/*
 * CORBA-bench-client.cc
 * Template to build a detailed CORBA benchmark
 */

#include <Padico/CORBA.h>
#include <Padico/COSNaming.h>
#include <Padico/Module.h>

#include "CORBA-bench.h"

#include <fstream.h>
#include <iomanip>
#include <algorithm>
#include <deque>
#include <string>

PADICO_MODULE_DECLARE(CORBA_bench_client);
PADICO_MODULE_INIT(corba_bench_init);
//PADICO_MODULE_RUN(mini_bench);
PADICO_MODULE_RUN(paco_bench);

extern "C" {
#include "timing.h"
}

#define RESULT_PATH  "/tmp/"

static Padico::ORB_ptr                orb;
static CosNaming::NamingContext_var   naming_context;
static Bench_server_ptr               bench_server;
static CosNaming::Name                bench_server_name;


struct bench_result_t
{
  double  total_time;
  double  min_time, max_time;
  long    data_size;
  double  avg_time;
  double  bw;
};

typedef deque<bench_result_t> bench_result_list_t;

struct bench_in_t
{
  long    size;  // in arbitrary unit
  int     count;
};

typedef deque<bench_in_t> bench_in_list_t;

bench_in_list_t*MakeLogNList(int min, int max, int count, double N)
{
  assert((min >= 1) && (max >= min));
  bench_in_list_t*bi_list = new bench_in_list_t;
  for(int i = min; i <= max; i = (int)(i*N) )
    {
      bench_in_t bi;
      bi.count = count;
      bi.size = i;
      bi_list->push_front(bi);
    }
  return bi_list;
}

bench_in_list_t*MakeLogNListReverse(int min, int max, int count, double N)
{
  assert((min >= 1) && (max >= min));
  bench_in_list_t*bi_list = new bench_in_list_t;
  for(int i = max; i > min; i = (int)(i/N) )
    {
      bench_in_t bi;
      bi.count = count;
      bi.size = i;
      bi_list->push_front(bi);
    }
  return bi_list;
}

bench_in_list_t*MakeLog2List(int min, int max, int count)
{
  return MakeLogNList(min, max, count, 2);
}

bench_in_list_t*ShuffleList(bench_in_list_t*bi_list)
{
  random_shuffle(bi_list->begin(), bi_list->end());
  return bi_list;
}   

class Bench_client_impl;
// class Bench
// base virtuelle de tous les Bench*

template<class CorbaType_t> class Bench
{
protected:
#ifdef UGO_ORB_PADICO_OMNIORB4
  typedef void (_objref_Bench_server::*BenchService_t)(const CorbaType_t&); 
#endif
#ifdef UGO_ORB_MICO
  typedef void (Bench_server::*BenchService_t)(const CorbaType_t&);
#endif
  char*name;
  Bench_server_ptr bench_server;
  BenchService_t service;
  static CorbaType_t msg;
public:
  Bench(char*n, Bench_server_ptr bs, BenchService_t s):
    name(n), bench_server(bs), service(s){}
  virtual void   run(bench_in_list_t*);
protected:
  virtual void   the_test(bench_in_t*, bench_result_t*) = 0;
  virtual void   init_data(int current_size) = 0;
  virtual double send_data() = 0;
};

template<class CorbaType_t> CorbaType_t Bench<CorbaType_t>::msg;

// class BenchSeq
// on suppose la declaration IDL :
// typedef sequence<BaseType_t> SeqType_t;

template<class SeqType_t, class BaseType_t> class BenchSeq : 
  public Bench<SeqType_t>
{
public:
  static void init_scalaire(BaseType_t*elem, int i)
  {
    *elem = i;
  }
public:
  typedef void (*init_elem_t)(BaseType_t*, int);
  BenchSeq(char*n,  Bench_server_ptr bs, BenchService_t s, init_elem_t f = &init_scalaire):
    Bench<SeqType_t>(n, bs, s), init_elem(f){};
protected:
  init_elem_t  init_elem;
  virtual void   the_test(bench_in_t*, bench_result_t*);
  virtual double send_data();
  virtual void   init_data(int current_size)
  {
    Bench<SeqType_t>::msg.length(current_size);
    for(int i = 0; i < current_size; i++)
      init_elem(&Bench<SeqType_t>::msg[i], i);
  }
};

// class BenchSeqStruct

template<class SeqType_t, class StructType_t> class BenchSeqStruct : 
  public BenchSeq<SeqType_t, StructType_t>
{
public:
  BenchSeqStruct(char*n, Bench_server_ptr bs, BenchService_t s):
    BenchSeq<SeqType_t, StructType_t>(n, bs, s, &init_struct){}
private:
  static void init_struct(StructType_t*elem, int i)
  {
    elem->a = i;
  }
};

// class BenchTab

template<class SeqType_t, class ArrayType_t, class BaseType_t, int tab_size> class BenchTab : 
  public BenchSeq<SeqType_t, ArrayType_t>
{
public:
  BenchTab(char*n, Bench_server_ptr bs, BenchService_t s):
    BenchSeq<SeqType_t, ArrayType_t>(n, bs, s, &init_array){}
protected:
  static void init_array(ArrayType_t&elem, int i)
  {
    for(int j=0; j<tab_size; j++)
      elem[j] = i + j;
  }
};


// class BenchDynSeq

template<class SeqType_t, class BaseType_t> class BenchDynSeq :
  public BenchSeq<SeqType_t, BaseType_t>
{
  char*service_name;
public:
  BenchDynSeq(char*n, Bench_server_ptr bs, char*s):
    BenchSeq<SeqType_t, BaseType_t>(n, bs, NULL), service_name(s){}
  virtual double send_data();
};

// Bench<>::run()

template<class CorbaType_t>
void Bench<CorbaType_t>::run(bench_in_list_t*bi_list)
{
  ofstream data_file_bw((string(RESULT_PATH)+string(name)+string(".bw")).c_str());
  ofstream data_file_time((string(RESULT_PATH)+string(name)+string(".t")).c_str());
  
  cout << "----- Starting benchmark: " << name << " -----" << endl;
  try {
    bench_result_list_t br_list;
    
    while(!bi_list->empty())
      {
	bench_in_t     bi = bi_list->front();
	bench_result_t br;
	
	init_data(bi.size);
	the_test(&bi, &br);
	br_list.push_front(br);
	cout << " -- " << br.data_size << " bytes: " << br.bw << " MB/s ";
	cout << " avg time " << setprecision(6) << br.avg_time << endl;
	data_file_bw << br.data_size << " " << br.bw << endl;
	data_file_time << br.data_size << " " << br.avg_time << " " << br.min_time << " " << br.max_time << endl;
	bi_list->pop_front();
	
      }
  } 
  catch(...) {
    cerr << "Exception during the test" << endl;
  }
  data_file_bw.close();
  data_file_time.close();
}

// BenchSeq<>::the_test()

template<class SeqType_t, class BaseType_t>
void BenchSeq<SeqType_t, BaseType_t>::the_test(bench_in_t*bi, bench_result_t*br)
{
  br->total_time = 0;
  br->min_time   = -1;
  br->max_time   = -1;
  for(int i = 0; i < bi->count; i++)
    {
      double c_time = send_data();
      br->total_time += c_time;
      if((br->min_time<0)||(c_time<br->min_time))
        br->min_time = c_time;
      if((br->max_time<0)||(c_time>br->max_time))
        br->max_time = c_time;
    }
  br->data_size  = bi->size * sizeof(BaseType_t);
  br->avg_time   = br->total_time / bi->count;
  br->bw         = br->data_size / br->avg_time;
}

// BenchSeq<>::send_data()

template<class SeqType_t, class BaseType_t>
double BenchSeq<SeqType_t, BaseType_t>::send_data()
{
  timing_t tick_start, tick_completed;
  timing_get_tick(&tick_start);
#ifdef UGO_ORB_PADICO_OMNIORB4
  (bench_server->*service)(Bench<SeqType_t>::msg);
#endif
#ifdef UGO_ORB_MICO
  (bench_server->*service)(Bench<SeqType_t>::msg);
#endif
  timing_get_tick(&tick_completed);
  return timing_usec(tick_start, tick_completed);
}

// BenchDynSeq<>::send_data()

template<class SeqType_t, class BaseType_t>
double BenchDynSeq<SeqType_t, BaseType_t>::send_data()
{
  timing_t tick_start, tick_completed;
  timing_get_tick(&tick_start);
  CORBA::Request_ptr req = (bench_server->in())->_request(service_name); // char*service !!!
  req->add_in_arg() <<= (Bench<SeqType_t>::msg);
  req->invoke();
  timing_get_tick(&tick_completed);
  return timing_usec(tick_start, tick_completed);
}

#ifdef UGO_ORB_PADICO_OMNIORB4
#define RUN_SEQ(TYPE, COUNT, MIN_SIZE, MAX_SIZE, LOG) \
 { BenchSeq<Seq##TYPE##_t, CORBA::##TYPE> \
     bench_seq##TYPE("Seq" #TYPE, bench_server, &_objref_Bench_server::BenchSeq##TYPE, &BenchSeq<Seq##TYPE##_t,CORBA::##TYPE>::init_scalaire); \
   bench_in_list_t*bi_list = MakeLogNListReverse(MIN_SIZE, MAX_SIZE, COUNT, LOG); \
   bench_seq##TYPE##.run(bi_list); \
   delete bi_list; }
  
#define RUN_DYN(TYPE, COUNT, MIN_SIZE, MAX_SIZE, LOG) \
 { BenchDynSeq<Seq##TYPE##_t, CORBA::##TYPE> \
     bench_dyn_seq##TYPE("DynSeq" #TYPE, bench_server, "BenchSeq" #TYPE); \
  bench_in_list_t*bi_list = MakeLogNListReverse(MIN_SIZE, MAX_SIZE, COUNT, LOG); \
  bench_dyn_seq##TYPE##.run(bi_list); }
#endif

#ifdef UGO_ORB_MICO
#define RUN_SEQ(TYPE, COUNT, MIN_SIZE, MAX_SIZE, LOG) \
 { BenchSeq<Seq##TYPE##_t, CORBA::##TYPE> \
     bench_seq##TYPE("Seq" #TYPE, bench_server, &Bench_server::BenchSeq##TYPE); \
   bench_in_list_t*bi_list = ShuffleList(MakeLogNListReverse(MIN_SIZE, MAX_SIZE, COUNT, LOG)); \
   bench_seq##TYPE##.run(bi_list); \
   delete bi_list; }

#define RUN_DYN(TYPE, COUNT, MIN_SIZE, MAX_SIZE, LOG) \
 { BenchDynSeq<Seq##TYPE##_t, CORBA::##TYPE> \
     bench_dyn_seq##TYPE("DynSeq" #TYPE, bench_server, "BenchSeq" #TYPE); \
  bench_in_list_t*bi_list = MakeLog2List(MIN_SIZE, MAX_SIZE, COUNT); \
  bench_dyn_seq##TYPE##.run(bi_list); }
#endif

#define RUN_BENCH(TYPE, COUNT, MIN_SIZE, MAX_SIZE) \
  { RUN_SEQ(TYPE, COUNT, MIN_SIZE, MAX_SIZE, 2); \
    RUN_ONEWAY(TYPE, COUNT, MIN_SIZE, MAX_SIZE,2); \
    RUN_DYN(TYPE, COUNT, MIN_SIZE, MAX_SIZE,2); }

void full_bench(int argc, char**argv)
{
  RUN_SEQ(Char,       50, 1, 1000000, 1.5);
  RUN_SEQ(Octet,      50, 1, 1000000, 1.5);
  RUN_SEQ(Short,      50, 1, 500000,  1.5);
  RUN_SEQ(Long,       50, 1, 250000,  1.5);
  RUN_SEQ(Float,      50, 1, 250000,  1.5);
  RUN_SEQ(Double,     50, 1, 100000,  1.5);
}

void mini_bench(int argc, char**argv)
{
  RUN_SEQ(Long, 500, 1, 5000000, 1.5);
}

class PacoBench : public Bench<SeqPaco_t>
{
public:
  PacoBench( Bench_server_ptr bs) : Bench<SeqPaco_t>("Paco", bs, &_objref_Bench_server::BenchSeqPaco) {};
  virtual void   the_test(bench_in_t*bi, bench_result_t*br)
  {
    br->total_time = 0;
    br->min_time   = -1;
    br->max_time   = -1;
    for(int i = 0; i < bi->count; i++)
      {
	double c_time = send_data();
	br->total_time += c_time;
	if((br->min_time<0)||(c_time<br->min_time))
	  br->min_time = c_time;
	if((br->max_time<0)||(c_time>br->max_time))
	  br->max_time = c_time;
      }
    br->data_size  = bi->size * sizeof(CORBA::Long);
    br->avg_time   = br->total_time / bi->count;
    br->bw         = br->data_size / br->avg_time;
  }
  virtual double send_data()
  {
    timing_t tick_start, tick_completed;
    try
      {
	timing_get_tick(&tick_start);
	bench_server->BenchSeqPaco(Bench<SeqPaco_t>::msg);
	timing_get_tick(&tick_completed);
      }
    catch(...)
      {
	cout << " Exception in PacoBench::send_data()" << endl;
      }
    return timing_usec(tick_start, tick_completed);
  }
  virtual void   init_data(int current_size)
  {
    try
      {
	DD_t dd;
	dd.proc = 1;
	dd.total = 18;
	dd.global.length(1);
	dd.global[0] = 3;
	dd.local2.length(1);
	dd.local2[0] = 18;
	msg.toto <<= dd;
	msg.data.length(current_size);
	for(int i = 0; i < current_size; i++)
	  msg.data[i] = i;
      }
    catch(...)
      {
	cout << " Exception in PacoBench::init_data()" << endl;
      }
  }
};

int paco_bench(int argc, char**argv)
{
  PacoBench paco(bench_server);
  bench_in_list_t*bi_list = MakeLogNListReverse(1, 100000, 20, 2);
  paco.run(bi_list);
  delete bi_list;

  RUN_SEQ(Long, 20, 1, 100000, 2);
  cout << "----- Ok. -----" << endl; 
  return 0;
}



//********************
//     main
//********************


int corba_bench_init(void)
{
  timing_init();
  
  orb = Padico::ORB_init();
  
  try
    {
      CORBA::Object_var ns_ref = 
	orb->resolve_initial_references ("NameService");
      naming_context = CosNaming::NamingContext::_narrow (ns_ref);
      
      bench_server_name.length (1);
      bench_server_name[0].id = CORBA::string_dup("default");
      bench_server_name[0].kind = CORBA::string_dup("BenchServer");
      
      CORBA::Object_var server_obj = naming_context->resolve(bench_server_name);
      bench_server = Bench_server::_narrow(server_obj);
    }
  catch(...)
    {
      padico_warning("Exception while starting benchmark client.");
      padico_warning("Is the benchmark server running?");
      return 1;
    }
  return 0;
}



