/* CORBA-vsock-bench.c
 * @deprecated
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <netdb.h>
#include "Padico-Puk.h"
#include "Padico-PadicoTM.h"
#include "Padico-VSock.h"
#include "Padico-na-Cluster.h"
extern "C" {
#include "timing.h"
}
#include "Padico-CORBA.h"
#include "Padico-Naming.h"

#include "Padico-Module.h"

#include "cluster-full.h"

#define DEFAULT_PATH  "/tmp/"
static char*path = DEFAULT_PATH;

#define DEFAULT_MAX_LEN        4*1024*1024
static long maxlen = DEFAULT_MAX_LEN;

#define DEFAULT_ROUNDTRIP  50
static int num_roundtrip = DEFAULT_ROUNDTRIP;

static char*server;

#define DEFAULT_PORT   5050
static int port = DEFAULT_PORT;
static int rank;
static int nproc;

static struct sockaddr_in local_name;
static socklen_t          local_name_size  = sizeof(local_name);
static struct sockaddr_in remote_name;
static socklen_t          remote_name_size = sizeof(remote_name);
static padico_vsock_t     bench_fd;

static void the_bench_vsock();
static void bench_init_vsock();
static void the_bench_corba();
static void bench_init_corba();

static int corba_bench_finish = 0;

class ClusterFullBench_impl : public POA_ClusterFullBench
{
public:
  void RecvSeqOctet(ClusterFullBench::SeqOctet_t &msg)
  {
  }
  void LatencyTest(void)
  {
  }
  void Finish(void)
  {
    corba_bench_finish = 1;
  }
};

static ClusterFullBench_impl*         bench_servant;
static ClusterFullBench_ptr           bench_ref = NULL;
static CORBA::ORB_ptr                 orb;
static PortableServer::POA_var        poa;
static PortableServer::POAManager_var poa_manager;
static CosNaming::NamingContext_var   naming_context;
static PortableServer::ObjectId_var   bench_server_oid;
static CosNaming::Name                bench_server_name;

PADICO_DECLARE_INIT(
{
  char*c;

  timing_init();

  rank = padico_cluster_rank();
  nproc = padico_cluster_size();
  server = padico_cluster_nodename(0);
  padico_print("*** self = %s\n", padico_module_self_name());
  c = padico_getattr("path");
  if(c) path = c;
  c = padico_getattr("maxlen");
  if(c) maxlen = atoi(c);
  c = padico_getattr("roundtrip");
  if(c) num_roundtrip = atoi(c);
  c = padico_getattr("server");
  if(c) server = c;
  c = padico_getattr("port");
  if(c) port = atoi(c);
  padico_print("BENCH: [path]      = %s\n", path);
  padico_print("BENCH: [maxlen]    = %d\n", maxlen);
  padico_print("BENCH: [roundtrip] = %d\n", num_roundtrip);
  padico_print("BENCH: [server]    = %s\n", server);
  padico_print("BENCH: [port]      = %d\n", port);
  bench_init_vsock();
  bench_init_corba();
});

PADICO_DECLARE_RUN(argc, argv,
{
  the_bench_vsock();
  padico_print("BENCH: VSock completed\n");
  the_bench_corba();
  padico_print("BENCH: completed!\n");
});

PADICO_DECLARE_FINALIZE(
{
   padico_print("BENCH: finalizing\n");
});

static void bench_init_vsock()
{
  if(rank == 0)
    {
      padico_vsock_t fd;
      int rc;
      fd = padico_socket(); /* TODO: changer -> (AF_INET, SOCK_STREAM, 0) */
      local_name.sin_family = AF_INET;
      local_name.sin_port  = port;
      rc = padico_bind(fd, (struct sockaddr*)&local_name, local_name_size);
      if(rc != 0)
	padico_fatal("cannot bind on port %d\n", port);
      /*      padico_listen(fd, 2); */
      bench_fd = padico_accept(fd, (struct sockaddr*)&remote_name, &remote_name_size);
    }
  else
    {  
      struct hostent*he;
      int rc;
      bench_fd = padico_socket();
      he = gethostbyname(server);
      memcpy(&remote_name.sin_addr, he->h_addr_list[0], sizeof(remote_name.sin_addr));
      remote_name.sin_family = AF_INET;
      remote_name.sin_port  = port;
      rc = padico_connect(bench_fd, (struct sockaddr_in*)&remote_name, remote_name_size);
      if(rc != 0)
	padico_fatal("cannot connect fd %d\n", bench_fd);
    }
}


static void the_bench_vsock()
{
  FILE*flog_time;
  FILE*flog_bw;
  int lenbuf;
  char *buffer;
  double bw_MBps;
  timing_t tick_start, tick_end;
  double total_time_usec;
  double avg_time_usec;
  int i;

 if(rank == 0)
    {
      padico_print("VSock bench ----------------------------------------\n");
      chdir(path);
      flog_time = fopen("VSock.t", "w");
      flog_bw   = fopen("VSock.bw", "w");
    }
  if ( (buffer = (char *) malloc((unsigned) maxlen)) == (char *) NULL)
    {
      padico_warning("process %d could not allocate buffer of size %d\n",rank, maxlen);
      return;
    }
  lenbuf = 1;
  while (lenbuf <= maxlen) 
    {
      total_time_usec = 0;
      for(i = 0; i < num_roundtrip ; i++)
	{
	  timing_get_tick(&tick_start);
	  if (rank == 0) 
	    {
	      /* TODO: loop for send/recv */
	      padico_send(bench_fd, buffer, lenbuf, 0);
	      padico_recv(bench_fd, buffer, lenbuf, 0);
	    }
	  else
	    {
	      padico_recv(bench_fd, buffer, lenbuf, 0);
	      padico_send(bench_fd, buffer, lenbuf, 0);
	    }
	  timing_get_tick(&tick_end);
	  total_time_usec += timing_usec(tick_start, tick_end);
	}
      avg_time_usec = total_time_usec / (double)(num_roundtrip * 2);
      if (avg_time_usec > 0)    /* rate is megabytes per second */
	bw_MBps = (double)(lenbuf / avg_time_usec);
      else
	bw_MBps = 0.0;
      if (rank == 0)
	{
	  padico_print("len=%d bytes, time=%f usec., bw=%f MB/s\n",
		       lenbuf, avg_time_usec, bw_MBps);
	  fprintf(flog_time, "%d %f\n", lenbuf, avg_time_usec);
	  fprintf(flog_bw, "%d %f\n", lenbuf, bw_MBps);
	}
      lenbuf *= 2;
    }
  if (rank == 0)
    {
      fclose(flog_time);
      fclose(flog_bw);
      padico_print("VSock bench ok -------------------------------------\n");
    }
  free(buffer);
 
}



static void bench_init_corba()
{
  orb = Padico_ORB_init();

  CORBA::Object_var poa_ref = orb->resolve_initial_references ("RootPOA");
  poa = PortableServer::POA::_narrow (poa_ref);
  poa_manager = poa->the_POAManager();
  poa_manager->activate();

  CORBA::Object_var ns_ref = 
    orb->resolve_initial_references ("NameService");
  naming_context = CosNaming::NamingContext::_narrow (ns_ref);

  bench_server_name.length (1);
  bench_server_name[0].id = CORBA::string_dup("default");
  bench_server_name[0].kind = CORBA::string_dup("ClusterFullBench");
  
  if(rank == 1)
    {
      bench_servant = new ClusterFullBench_impl;
      bench_server_oid = poa->activate_object(bench_servant);
      naming_context->rebind(bench_server_name, bench_servant->_this());
    }
  else
    {
      padico_tm_usleep(1000); /* on laisse le temps pour que le serveur s'enregistre... */
      CORBA::Object_var server_obj = naming_context->resolve(bench_server_name);
      bench_ref = ClusterFullBench::_narrow(server_obj);
    }
}


static void the_bench_corba()
{
  FILE*flog_time;
  FILE*flog_bw;
  int lenbuf;
  CORBA::Octet *buffer;
  double bw_MBps;
  timing_t tick_start, tick_end;
  double total_time_usec;
  double avg_time_usec;
  int i;

  if(rank == 0)
    {
      /* *** client */
      padico_print("CORBA bench ----------------------------------------\n");
      bench_ref->LatencyTest();
      chdir(path);
      flog_time = fopen("CORBA.t", "w");
      flog_bw   = fopen("CORBA.bw", "w");
      if ( (buffer = (CORBA::Octet*) malloc((unsigned) maxlen)) == (CORBA::Octet*) NULL)
	{
	  padico_warning("process %d could not allocate buffer of size %d\n",rank,maxlen);
	  return;
	}

      total_time_usec = 0;
      for(i = 0; i < num_roundtrip ; i++)
	{
	  timing_get_tick(&tick_start);
	  bench_ref->LatencyTest();
	  timing_get_tick(&tick_end);
	  total_time_usec += timing_usec(tick_start, tick_end);
	}
      avg_time_usec = total_time_usec / (double)(num_roundtrip * 2);
      padico_print("latency=%f usec.\n", avg_time_usec);

      lenbuf = 1;
      while (lenbuf <= maxlen) 
	{
	  ClusterFullBench::SeqOctet_t msg = ClusterFullBench::SeqOctet_t(maxlen, lenbuf, buffer);
	  total_time_usec = 0;
	  for(i = 0; i < num_roundtrip ; i++)
	    {
	      try
		{
		  timing_get_tick(&tick_start);
		  bench_ref->RecvSeqOctet(msg);
		  timing_get_tick(&tick_end);
		}
	      catch(...)
		{
		  tick_start = tick_end;
		}
	      total_time_usec += timing_usec(tick_start, tick_end);
	    }
	  avg_time_usec = total_time_usec / (double)(num_roundtrip * 2);
	  if (avg_time_usec > 0)
	    {
	      bw_MBps = (double)(lenbuf / avg_time_usec);
	      padico_print("len=%d bytes, time=%f usec., bw=%f MB/s\n",
			   lenbuf, avg_time_usec, bw_MBps);
	      fprintf(flog_time, "%d %f\n", lenbuf, avg_time_usec);
	      fprintf(flog_bw, "%d %f\n", lenbuf, bw_MBps);
	    }
	  else
	      padico_print("len=%d bytes, time=---, bw=---\n",lenbuf);
	  lenbuf *= 2;
	}
      fclose(flog_time);
      fclose(flog_bw);
      padico_print("CORBA bench ok -------------------------------------\n");
      bench_ref->Finish();
      free(buffer);
    }
  else
    {
      /* *** server */
      while(!corba_bench_finish)
	{
	  padico_tm_usleep(1000);
	}
    }
}
