/* 
 * timing.h
 */

#ifndef TIMING_H
#define TIMING_H

/* Ici on force l'architecture a X86 sans verification... :-/ */
#define  X86_ARCH

#include <sys/types.h>
#include <sys/time.h>

typedef union {
  unsigned long long tick;
  struct {
    unsigned long low, high;
  } sub;
  struct timeval timev;
} timing_t;

typedef struct {
  timing_t in_timing;
  timing_t out_timing;
  unsigned long long cumul_ticks;
  long cumul_size;
  long current_size;
} timing_avg_size_t;

typedef struct {
  timing_t last_tick;
  timing_t cumul_tick;
  long     nb_exec;
  double   avg_usec;
} timing_avg_t;

/* Hum hum... Here we suppose that X86ARCH => Pentium! */
#if defined(X86_ARCH) && !defined(FREEBSD_SYS)

#define TIMING_INTERNAL_GET_TICK(t)           __asm__ volatile("rdtsc" : "=a" ((t).sub.low), "=d" ((t).sub.high))

#define TIMING_GET_TICK(t) timing_get_tick(&(t))

#define TIMING_TICK_RAW_DIFF(t1, t2) ((t2).tick - (t1).tick)

#else

#define TIMING_INTERNAL_GET_TICK(t)  gettimeofday(&(t).timev, NULL)
#define TIMING_GET_TICK(t)           gettimeofday(&(t).timev, NULL)

#define TIMING_TICK_RAW_DIFF(t1, t2) ((t2.timev.tv_sec * 1000000L + t2.timev.tv_usec) - \
			       (t1.timev.tv_sec * 1000000L + t1.timev.tv_usec))

#endif

#define TIMING_TICK_DIFF(t1, t2)     (TIMING_TICK_RAW_DIFF(t1, t2) - timing_residual)

#define TIMING_EVENT(name) \
  { TIMING_GET_TICK(new_timing_event); timing_stat(name); TIMING_GET_TICK(last_timing_event); }

#define SILENT_TIMING_EVENT() \
  TIMING_GET_TICK(last_timing_event)

#define TIMING_AVG_IN(t_avg) \
  {  t_avg.nb_exec++; TIMING_GET_TICK(t_avg.last_tick); }

#define TIMING_AVG_OUT(t_avg, name, freq) \
  { timing_t tick; TIMING_GET_TICK(tick); \
    t_avg.cumul_tick.tick +=  tick.tick - t_avg.last_tick.tick; \
    t_avg.avg_usec = timing_tick2usec(t_avg.cumul_tick.tick)/t_avg.nb_exec; \
    if(t_avg.nb_exec%freq == 0){ fprintf(stderr, "T: %s total: %f average: %f for %d runs \n", name, timing_tick2usec(t_avg.cumul_tick.tick), t_avg.avg_usec, t_avg.nb_exec);} }

#define PROF_IN(LEN, SIZEOF) \
  static timing_avg_size_t _prof; \
  if(_prof.current_size != LEN) \
  { if(_prof.current_size != 0) \
    { double total_usec, Mbytes_per_sec; long long total_size_bytes, block_size_bytes; \
      total_usec = timing_tick2usec(_prof.cumul_ticks); \
      total_size_bytes =  _prof.cumul_size*SIZEOF; \
      block_size_bytes = _prof.current_size*SIZEOF; \
      Mbytes_per_sec = ((double) total_size_bytes) / total_usec; \
      fprintf(stdout, "%s # %lld %f \n   Total: %f usec for %lld bytes (block size %lld)\n", \
      __FUNCTION__, block_size_bytes, Mbytes_per_sec, \
      total_usec, total_size_bytes, block_size_bytes ); }; \
    _prof.cumul_size = 0; \
    _prof.cumul_ticks = 0; \
    _prof.current_size = LEN; } \
  TIMING_GET_TICK(_prof.in_timing);

#define PROF_OUT() \
  TIMING_GET_TICK(_prof.out_timing); \
  _prof.cumul_ticks += _prof.out_timing.tick - _prof.in_timing.tick; \
  _prof.cumul_size += _prof.current_size;  


extern void timing_init(void);

extern void timing_get_tick(timing_t*t);

extern double timing_tick2usec(long long t);
#define timing_usec(t1, t2) (timing_tick2usec(TIMING_TICK_DIFF(t1, t2)))

extern void timing_stat(char *name);

extern long long timing_residual;

extern timing_t new_timing_event, last_timing_event;


#endif /* TIMING_H */

