/** @file
 * @brief ID Packet Filter adapter- tester for VLink-PacketFilter
 */

#include <Padico/Module.h>
#include "PacketFilter.h"

PADICO_MODULE_DECLARE(PacketFilter_ID);
PADICO_MODULE_INIT(idpf_init);

/* ********************************************************* */
/* *** ID Packet Filter ************************************ */
/* ********************************************************* */


/** @defgroup PacketFilter_ID adapter: PacketFilter-ID- a packet filter that does nothing (identity function)
 * @note for testing purpose
 * @ingroup PadicoAdapter
 */

static void* idpf_instanciate(puk_instance_t assembly,
			      puk_context_t context);
static void idpf_destroy(void*state);
static void idpf_encode(void*, const void*, size_t, void**, size_t*);
static void idpf_decode(void*, const void*, size_t, void*, size_t*);

/** instanciation facet for PacketFilter-ID
 * @ingroup PacketFilter_ID
 */
const static struct puk_adapter_driver_s idpf_adapter_driver =
  {
    .instanciate = &idpf_instanciate,
    .destroy     = &idpf_destroy
  };
/** 'PacketFilter' facet for PacketFilter-ID
 * @ingroup PacketFilter_ID
 */
const static struct packetfilter_driver_s idpf_driver = 
  {
    .pf_encode = &idpf_encode,
    .pf_decode = &idpf_decode
  };


/* ********************************************************* */

struct id_state_s
{
  int dummy;
};
typedef struct id_state_s* id_state_t;

static void*idpf_instanciate(puk_instance_t assembly, puk_context_t context)
{
  const id_state_t state = padico_malloc(sizeof(struct id_state_s));
  return state;
}

static void idpf_destroy(void*state)
{
  padico_free(state);
}


/* 'out' allocated by encode function
 */
static void idpf_encode(void*state,
			const void*in, size_t in_size,
			void**out, size_t* out_size)
{
  void*p = padico_malloc(in_size);
  memcpy(p, in, in_size);
  *out = p;
  *out_size = in_size;
}

/* 'out' allocated by caller
*/
static void idpf_decode(void*state,
			const void*in, size_t in_size,
			void*out, size_t* out_size)
{
  assert(*out_size >= in_size);
  memcpy(out, in, in_size);
  *out_size = in_size;
}


static int idpf_init(void)
{
  padico_print("PacketFilter-ID: init\n");

  puk_component_declare("PacketFilter-ID",
			puk_component_provides("PadicoAdapter", "adapter", &idpf_adapter_driver),
			puk_component_provides("PacketFilter", "pf", &idpf_driver));
  return 0;
}
