AC_DEFUN([AC_PADICO_OUT_MODULES],
	 [ dnl --- Core/
	   out_mk="${out_mk} Core/Makefile Core/PadicoTM.map"
	   out_sh="${out_sh} Core/padico-marcelize"
	   dnl --- PadicoBoot/
	   out_mk="${out_mk} PadicoBoot/Makefile"
	   out_sh="${out_sh} PadicoBoot/padico-d \
 		  	     PadicoBoot/padico-launch"
	   dnl --- Services/
	   out_mk="${out_mk} Services/Makefile"
	   services=`cd ${padico_srcdir}/Services; find * -type d|grep -v CVS|tr '\n' ' '`
	   for service in ${services}; do
	     if test -r ${padico_srcdir}/Services/${service}/Makefile.in; then
	       out_mk="${out_mk} Services/${service}/Makefile"
	     fi
	     if test -r ${padico_srcdir}/Services/${service}/${service}.map.in; then
	       out_mk="${out_mk} Services/${service}/${service}.map"
	     fi
	   done
	   out_mk="${out_mk} PadicoTM.pc core_vars.mk"
	 ])

# ## check for Infiniband availbility
AC_DEFUN([AC_PADICO_INFINIBAND],
	 [ if test "x$IBHOME" != "x" ; then
	     ibverbs_CFLAGS="-I$IBHOME/include"
             if test "x${IBLIBPATH}" = "x" ; then
               IBLIBPATH=${IBHOME}/lib
             fi
	     ibverbs_LIBS="-Wl,-rpath,${IBLIBPATH} -L$IBLIBPATH -libverbs"
	   else
	     ibverbs_LIBS="-libverbs"
	   fi
	   AC_CHECK_HEADER(infiniband/verbs.h, 
	 	           [ HAVE_INFINIBAND=yes ],
	                   [ HAVE_INFINIBAND=no  ])
	   AC_SUBST(HAVE_INFINIBAND)
	   AC_SUBST(ibverbs_CFLAGS)
	   AC_SUBST(ibverbs_LIBS)
	 ])

# ## check for MX availbility
AC_DEFUN([AC_PADICO_MX],
	 [ if test "x$MX_DIR" = "x" ; then
	     if test -r /opt/mx/include/myriexpress.h ; then
	       MX_DIR=/opt/mx
	     fi
	   fi
	   if test "x$MX_DIR" != "x" ; then
 	     mx_CFLAGS="-I${MX_DIR}/include"
	     mx_LIBS="-Wl,-rpath,${MX_DIR}/lib -L${MX_DIR}/lib -lmyriexpress"
	   fi
	   AC_CHECK_HEADER(myriexpress.h, 
	 	           [ HAVE_MX=yes ],
	                   [ HAVE_MX=no  ])
	   AC_SUBST(HAVE_MX)
	   AC_SUBST(mx_CFLAGS)
	   AC_SUBST(mx_LIBS)
	 ])

# ## required and optional features
AC_DEFUN([AC_PADICOTM_FEATURES],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])

	   dnl - required headers

	   AC_CHECK_HEADERS( errno.h pthread.h sys/stat.h sys/wait.h, 
	     [],
	     AC_MSG_ERROR([Missing required header file]))
	

	   dnl - system-dependent headers

	   AC_CHECK_HEADER(poll.h, 
	     [ HAVE_POLL_H="yes"
	       AC_DEFINE(HAVE_POLL_H, 1, [Define if you have the <poll.h> header file.])],
	     [HAVE_POLL_H="no"])

	   AC_CHECK_HEADER(ifaddrs.h,
	     [ HAVE_IFADDRS_H="yes"
	       AC_DEFINE(HAVE_IFADDRS_H, 1, [Define if you have the <ifaddrs.h> header file.])],
	     [HAVE_IFADDRS_H="no"])

	   AC_CHECK_HEADER(sys/inotify.h,
	     [ HAVE_INOTIFY_H="yes"
	       AC_DEFINE(HAVE_INOTIFY_H, 1, [Define if you have the <sys/inotify.h> header file.])],
	     [HAVE_INOTIFY_H="no"])

	   AC_CHECK_HEADER(linux/netlink.h,
	     [ HAVE_LINUX_NETLINK_H="yes"
	       AC_DEFINE(HAVE_LINUX_NETLINK_H, 1, [Define if you have the <linux/netlink.h> header file.])],
	     [HAVE_LINUX_NETLINK_H="no"])

	   AC_CHECK_HEADER(linux/rtnetlink.h,
	     [ HAVE_LINUX_RTNETLINK_H="yes"
	       AC_DEFINE(HAVE_LINUX_RTNETLINK_H, 1, [Define if you have the <linux/rtnetlink.h> header file.])],
	     [HAVE_LINUX_RTNETLINK_H="no"])

	   AC_CHECK_HEADER(linux/sockios.h,
	     [ HAVE_LINUX_SOCKIOS_H="yes"
	       AC_DEFINE(HAVE_LINUX_SOCKIOS_H, 1, [Define if you have the <linux/sockios.h> header file.])],
	     [HAVE_LINUX_SOCKIOS_H="no"])

	   AC_CHECK_HEADER(linux/ethtool.h,
	     [ HAVE_LINUX_ETHTOOL_H="yes"
	       AC_DEFINE(HAVE_LINUX_ETHTOOL_H, 1, [Define if you have the <linux/ethtool.h> header file.])],
	     [HAVE_LINUX_ETHTOOL_H="no"])

	   AC_CHECK_HEADER(linux/if.h,
	     [ HAVE_LINUX_IF_H="yes"
	       AC_DEFINE(HAVE_LINUX_IF_H, 1, [Define if you have the <linux/if.h> header file.])],
	     [HAVE_LINUX_IF_H="no"],
	     [#include <netinet/in.h>])


	   dnl - optional headers
   
	   AC_CHECK_HEADER(graphviz/cgraph.h, 
	     [ HAVE_CGRAPH_H="yes"
	       HAVE_CGRAPH="yes"
	       AC_DEFINE(HAVE_CGRAPH_H, 1, [Define if you have the <graphviz/cgraph.h> header file.])],
	     [ HAVE_CGRAPH_H="no"
	       HAVE_CGRAPH="no" ])
	   AC_SUBST(HAVE_CGRAPH)
	   
	   AC_CHECK_HEADER(libssh2.h,
	     AC_CHECK_LIB(ssh2, libssh2_session_handshake,
	       [ HAVE_LIBSSH2_H=yes
	         AC_DEFINE(HAVE_LIBSSH2_H, 1, [Define if you have the <libssh2.h> header file.])],
	       [ HAVE_LIBSSH2_H=no ])
	   AC_SUBST(HAVE_LIBSSH2_H))

	     dnl - system-dependent symbols

	   AC_CHECK_FUNCS([ __fxstat ])
	 ])
