/** @file
 * @brief Internal definitions for use inside Puk only.
 * @ingroup Puk
 */

/** @defgroup Puk Core module: Puk -- PadicoTM micro-kernel
 * @author Alexandre Denis
 */


#ifndef PUK_INTERNALS_H
#define PUK_INTERNALS_H

#include "Puk.h"

/* include autoconf'ed configuration */
#include "puk_config.h"

/** @todo isn't there a better place to define PADICO? */
#ifndef PADICO
#  define PADICO
#endif

/* we need socklen_t */
#ifndef HAVE_SOCKLEN_T
typedef int socklen_t;
#endif


#include <stdio.h>
#include <stdlib.h>
#include <dlfcn.h>
#include <fcntl.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <stdarg.h>
#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>

__PUK_SYM_INTERNAL void padico_puk_debug_init(void);
__PUK_SYM_INTERNAL void padico_puk_builtin_init(void);
__PUK_SYM_INTERNAL void padico_puk_pkg_init(void);
__PUK_SYM_INTERNAL void padico_puk_lib_init(void);
__PUK_SYM_INTERNAL void padico_puk_monitor_init(void);
__PUK_SYM_INTERNAL void padico_puk_mod_init(void);
__PUK_SYM_INTERNAL void padico_puk_xml_init (void);
__PUK_SYM_INTERNAL void padico_puk_components_init(void);
__PUK_SYM_INTERNAL void padico_puk_binary_init(void);

__PUK_SYM_INTERNAL void padico_puk_debug_finalize(void);
__PUK_SYM_INTERNAL void padico_puk_xml_finalize(void);
__PUK_SYM_INTERNAL void padico_puk_components_finalize(void);
__PUK_SYM_INTERNAL void padico_puk_mod_finalize(void);

__PUK_SYM_INTERNAL padico_rc_t  puk_mod_open(puk_mod_t*mod, padico_string_t filename);
__PUK_SYM_INTERNAL puk_mod_t    puk_mod_new(const char*name, const char*driver_name);
__PUK_SYM_INTERNAL void         puk_mod_destroy(puk_mod_t m);
__PUK_SYM_INTERNAL void         puk_mod_builtin_tryload(puk_mod_t*mod, const char*mod_name);

/* lock Puk internal structures (xml/mod/components) */
__PUK_SYM_INTERNAL void __puk_lock_acquire(const char*func);
__PUK_SYM_INTERNAL void __puk_lock_release(const char*func);
#define puk_lock_acquire() __puk_lock_acquire(__FUNCTION__)
#define puk_lock_release() __puk_lock_release(__FUNCTION__)

/* ** attrs */

struct puk_attr_s
{
  const char* label;
  const char* value;
};
typedef struct puk_attr_s*puk_attr_t;

/* ** units */
PUK_VECT_TYPE(puk_unit, struct puk_unit_s);

/* ** mod_refs */
PUK_VECT_TYPE(puk_mod, struct puk_mod_s*);

/* ** module */

struct puk_mod_s
{
  char*                     mod_name;    /**< name of the module */
  puk_component_t           driver;      /**< loader used to manage this module */
  struct puk_unit_vect_s    units;       /**< units- content of the module */
  puk_hashtable_t           attrs;       /**< attributes of the module */
  puk_mod_t                 inherits;    /**< ancestor for attributes inheriting */
  struct puk_mod_vect_s     requires;    /**< modules used by this module */
  struct padico_modID_vect_s unresolved_deps; /**< unresolved requirements */
  int                       used_by;     /**< number of modules using this one */
  int                       ref_count;   /**< number of references (explicit 'load') */
  int                       managed_instances; /**< number of components instances whose home is this module */
  int                       own_instances; /**< number of components that have this module as owner and home */
};

#endif /* PUK_INTERNALS_H */
