
AC_DEFUN([AC_PADICO_OUT_PUK],
	 [ dnl --- Puk/
	   out_mk="${out_mk} Makefile Puk.pc Puk.gdbinit"
	   dnl --- Puk/ModulesTools/
	   out_mk="${out_mk} ModulesTools/Makefile ModulesTools/padico-core-wrapper.sh"
	   out_sh="${out_sh} ModulesTools/padico-mkmod ModulesTools/padico-cc ModulesTools/padico-cdep ModulesTools/padico-config"
	 ])

AC_DEFUN([AC_PADICO_OUT_PUK_BUILD],
	 [
	   common_vars_mk=${padico_builddir}/common_vars.mk
	   AC_SUBST_FILE(common_vars_mk)
	   out_mk="common_vars.mk $out_mk"

	   common_rules_mk=${padico_builddir}/common_rules.mk
	   AC_SUBST_FILE(common_rules_mk)
	   out_mk="common_rules.mk $out_mk"
	  ])


# ## check how to disable std lib linkage
# used by padico-mkmod
AC_DEFUN([AC_PADICO_NOSTDLIB],
	 [ AC_REQUIRE([AC_PADICO_LINKER])
	   AC_MSG_CHECKING([for nostdlib ld flag])
	   case ${LD} in
	     *c++*|*g++*|*gcc*|*clang*|*icc*) LDNOSTDLIB="";; # LDNOSTDLIB="-nostdlib";;
	     'CC')  LDNOSTDLIB="-z muldefs";;
	     *) AC_MSG_ERROR([Unknown C++ linker: ${LD}. Stop.])
	   esac
	   AC_MSG_RESULT([${LDNOSTDLIB}])
	   AC_SUBST(LDNOSTDLIB)
	 ])



# ## how to convert a static library into a shared object file
# used by padico-mkmod
AC_DEFUN([AC_PADICO_ATOSO],
	 [ AC_REQUIRE([AC_PADICO_COMPILER]) 
	   AC_MSG_CHECKING([ whether static libraries can be converted into dynamic libraries ]) 
	   libm_a=`${CC} -print-file-name=libm.a`
	   if test -r ${libm_a} ; then
	     _LOCALLIBA=${libm_a}
	   else
	     _LOCALLIBA=`echo /usr/lib/lib*.a | cut -f 1 -d ' '`
	     if test ! -r ${_LOCALLIBA} ; then
	       AC_MSG_WARN([ Unable to find a lib.a in /usr/lib ])
	     fi
	   fi
	   _LOCALLIBSO=test.so
	   atoso_flags_gnu="-shared -export-dynamic -whole-archive -fPIC"
	   atoso_flags_bsd="-G z allextract"
	   ATOSO_OK="yes"
	   if ( ld ${atoso_flags_gnu} ${_LOCALLIBA} -o ${_LOCALLIBSO} 1> /dev/null 2>&1 ); then
	     ATOSO="ld ${atoso_flags_gnu}";
	   elif ( gnu-ld ${atoso_flags_gnu} ${_LOCALLIBA} -o ${_LOCALLIBSO} 1> /dev/null 2>&1 ); then
	     ATOSO="gnu-ld ${atoso_flags_gnu}";
	   elif ( ld ${atoso_flags_bsd} ${_LOCALLIBA} -o ${_LOCALLIBSO}  1> /dev/null 2>&1 ); then
	     ATOSO="ld ${atoso_flags_bsd}";
	   else
	     ATOSO="";
	     ATOSO_OK="no";
	   fi
	   if test -r ${_LOCALLIBSO} ; then rm ${_LOCALLIBSO}; fi
	   AC_MSG_RESULT([${ATOSO_OK}: ${ATOSO} ])
	   AC_SUBST(ATOSO)
	   AC_SUBST(ATOSO_OK)
	 ])

# ## dynamic objects
AC_DEFUN([AC_PADICO_DSO],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_MSG_CHECKING([for dynamic object (DSO) linker])
	   DSO_LD="${CC} ${CPICFLAGS} ${DSO_FLAGS}"
	   AC_MSG_RESULT([${DSO_LD}])
	   AC_SUBST(DSO_LD)

	   AC_MSG_CHECKING([for DSO extension])
	   AC_MSG_RESULT([${DSO_EXT}])
	   AC_SUBST(DSO_EXT)
	   AC_DEFINE_UNQUOTED(DSO_EXT, "${DSO_EXT}",
	     [Extension for bundles])

	   AC_MSG_CHECKING([for symbol prefix])
	   AC_MSG_RESULT([${SYMBOL_PREFIX}])
	   AC_SUBST(SYMBOL_PREFIX)
	   AC_DEFINE_UNQUOTED(SYMBOL_PREFIX, "${SYMBOL_PREFIX}",
	     [prefix prepended to symbols by the ABI (usually "_")])
	 ])

# ## required and optional features
AC_DEFUN([AC_PUK_FEATURES],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   dnl - required headers
	   AC_CHECK_HEADERS(dlfcn.h setjmp.h errno.h sys/stat.h sys/wait.h, 
	     [],
	     AC_MSG_ERROR([Missing required header file]))
	
	   AC_CHECK_HEADER(elf.h, 
	     [ HAVE_ELF_H="yes"
	       AC_DEFINE(HAVE_ELF_H, 1, [Define if you have the <elf.h> header file.])],
	     [HAVE_ELF_H="no"])

	   AC_CHECK_HEADER(stdint.h, 
	     [ HAVE_STDINT_H="yes"
	       AC_DEFINE(HAVE_STDINT_H, 1, [Define if you have the <stdint.h> header file.])],
	     [HAVE_STDINT_H="no"])

	   AC_CHECK_FUNCS([ setenv ])
	 ])

