/** @file Miscellaneous oracles for number of args of variadic functions
 * @ingroup PukABI
 */

#include <stdio.h>
#include <unistd.h>
#include <poll.h>
#include <fcntl.h>
#include <string.h>

#include "Puk-ABI.h"

/** return the number of arguments for fcntl */
int puk_abi_oracle__fcntl(int cmd)
{
  int o = -1;
  switch(cmd)
    {
    case F_DUPFD:
    case F_GETFD:
    case F_GETFL:
      o = 0;
      break;

    case F_SETFD:
    case F_SETFL:
      o = 1;
      break;
      
    case F_GETLK:
    case F_SETLK:
    case F_SETLKW:
#if defined F_GETLK64 && F_GETLK64 != F_GETLK
    case F_GETLK64:
    case F_SETLK64:
    case F_SETLKW64:
#endif
      o = 2;
      break;
      
    default:
      puk_abi_print("PukABI: warning- fcntl() called with unsupported option %d", cmd);
      o = 2;
      break;
    }
  return o;
}

static inline int strprefix(const char*s1, const char*s2)
{
  return strncmp(s1, s2, strlen(s2));
}

/** return the kind of fd to use according to filename */
int puk_abi_oracle__open_kind(const char*path)
{
  if( (strprefix(path, "/dev/") == 0) ||
      (strprefix(path, "/sys/") == 0) ||
      (strprefix(path, "/proc/") == 0) ||
      (strprefix(path, "/run/shm/") == 0) )
    {
      puk_abi_trace("PukABI: use **real** for path %s", path);
      return 1;
    }
  else
    {
      puk_abi_trace("PukABI: use default for path %s", path);
      return 0;
    }
}
/** return whether to use 3 parameters (creation mode) or 2 for open() */
int puk_abi_oracle__open_flags(int oflag)
{
  return(oflag & O_CREAT);
}
