
AC_DEFUN([AC_PADICO_COMPILER],
	 [

dnl - save current state
MYCPPFLAGS="$CPPFLAGS"
MYCFLAGS="$CFLAGS"
MYCXXFLAGS="$CXXFLAGS"
MYLDFLAGS="$LDFLAGS"

dnl - standard compiler checks
AC_PROG_CC
AC_PROG_CPP
AC_PROG_CXX
AC_PROG_CXXCPP

dnl --- compute flags depending on the OS type

case `uname -s` in
  SunOS)
    binary_format=elf
    dnl - dynamic libraries
    LDFLAGS=
    SYMBOL_PREFIX=""
    DYNLIB_PREFIX="lib"
    DYNLIB_EXT=".so"
    DYNLIB_FLAGS="-shared"
    DSO_EXT=".so"
    DSO_FLAGS="-shared"
    case ${CC} in
      gcc*) CFLAGS="-g -Wall"
            CPICFLAGS="-fPIC"
	    CDEPFLAGS="-MM";;
      cc)   CFLAGS="-g"
	    CPICFLAGS="-Kpic"
	    CDEPFLAGS="-xM1";;
    esac
    case ${CXX} in
      c++|g++*) CXXFLAGS="-g"
		CXXPICFLAGS="-fPIC"
               	CXXDEPFLAGS="-MM";;
           CC)  CXXFLAGS="-g"
		CXXPICFLAGS="-Kpic"
	        CXXDEPFLAGS="-xM1"
		CXXSHFLAGS="-G";;
    esac
  ;;
  Linux|*BSD)
    binary_format=elf
    LDFLAGS="-rdynamic"
    SYMBOL_PREFIX=""
    DYNLIB_PREFIX="lib"
    DYNLIB_EXT=".so"
    DYNLIB_FLAGS="-shared -rdynamic"
    DSO_EXT=".so"
    DSO_FLAGS="-shared -rdynamic"
    OPT_LDFLAGS="-Wl,-O1"
    dnl gcc
    BASE_CFLAGS="-Wall -pipe"
    DEBUG_CFLAGS="-g"

    gcc_version="`${CC} -dumpversion`"
    case $gcc_version in
       2.*|3.*|4.0*|4.1*)
	OPT_CFLAGS="-O3"
        ;;
       *) dnl  -march=native is only available since gcc 4.2
	OPT_CFLAGS="-O3 -march=native"
	;;
    esac
	
    CPICFLAGS="-fPIC"
    CDEPFLAGS="-MM"
    dnl g++
    BASE_CXXFLAGS=""
    DEBUG_CXXFLAGS="-g"
    OPT_CXXFLAGS="-O2"
    CXXPICFLAGS="-fPIC"
    CXXDEPFLAGS="-MM"
  ;;
  IRIX64)
    binary_format=elf
    dnl - dynamic libraries
    LDFLAGS="-rdynamic"
    SYMBOL_PREFIX="" 
    DYNLIB_PREFIX="lib"
    DYNLIB_EXT=".so"
    DYNLIB_FLAGS="-shared"
    DSO_EXT=".so"
    DSO_FLAGS="-shared"
    dnl gcc-specific flags... but no other compiler supported
    CFLAGS="-Wall -Dirix"
    CPICFLAGS="-fPIC"
    CDEPFLAGS="-MM"
    CDEBUGFLAGS="-g"
    COPTFLAGS="-O4"
    dnl g++ flags...
    CXXFLAGS="-Wall -Dirix"
    CXXPICFLAGS="-fPIC"
    CXXDEPFLAGS="-MM"
    CXXDEBUGFLAGS="-g"
    CXXOPTFLAGS="-O2" dnl it's *really* -O2, not -O4. Else, it doesn't work with omniORB 3
  ;;
  Darwin)
    binary_format="Mach-O"
    dnl - dynamic libraries
    LDFLAGS=""
    SYMBOL_PREFIX="_"
    DYNLIB_PREFIX="lib"
    DYNLIB_EXT=".dylib"
    DYNLIB_FLAGS="-flat_namespace -undefined suppress -fno-common -dynamiclib"
    DSO_EXT=""
    DSO_FLAGS="-flat_namespace -undefined suppress -bundle"
    dnl - gcc
    CFLAGS="-g -Wall -traditional-cpp -fno-common -Wno-long-double"
    CPICFLAGS="-fPIC"
    CDEPFLAGS="-MM"
    dnl - g++
    CXXFLAGS="-g -Wall -traditional-cpp"
    CXXPICFLAGS="-fPIC"
    CXXDEPFLAGS="-MM"
  ;;
  *) # What is this f$*k!@ OS? :-)
    AC_MSG_ERROR([Unknown OS/platform: `uname -s`])
  ;;
esac

# user specified and standard flags
CPPFLAGS="$MYCPPFGLAGS -D_REENTRANT -D_GNU_SOURCE -DPADICO"
LDFLAGS="$MYLDFLAGS $LDFLAGS"
BASE_CFLAGS="$MYCFLAGS $BASE_CFLAGS"
BASE_CXXFLAGS="$MYCXXFLAGS $BASE_CXXFLAGS"

dnl --- pic/debug/opt flags for C/C++
AC_SUBST(CPPFLAGS)
AC_SUBST(CPICFLAGS)
AC_SUBST(CXXPICFLAGS)
AC_SUBST(DEBUG_CFLAGS)
AC_SUBST(BASE_CFLAGS)
AC_SUBST(BASE_CXXFLAGS)
AC_SUBST(OPT_CFLAGS)
AC_SUBST(DEBUG_CXXFLAGS)
AC_SUBST(OPT_CXXFLAGS)
AC_SUBST(OPT_LDFLAGS)
AC_SUBST(binary_format)

         ])


AC_DEFUN([AC_PADICO_GCC_VERSION],
	 [ AC_REQUIRE([AC_PADICO_VERSION])
	   AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_MSG_CHECKING([for gcc version])
	   if test "x$GCC" != "xyes"; then
	     AC_MSG_ERROR([ $PACKAGE_STRING requires GNU C compiler.])
	   else
	     gcc_version="`${CC} -dumpversion`"
	     case $gcc_version in
	       2.9*|3.0.4|3.1*|3.2*|3.3*|3.4*|4.*)
	         AC_MSG_RESULT([ ${gcc_version} ok])
	       ;;
	       2.7*|2.8*|3.0*|2.96*)
	         AC_MSG_RESULT([ ${gcc_version} not supported-- warning: gcc version not supported])
	       ;;
	       *)
	         AC_MSG_RESULT([ ${gcc_version} unknown-- report how it works!])
	       ;;
	     esac
	   fi
	  ])

AC_DEFUN([AC_PADICO_GXX_VERSION],
	 [ AC_REQUIRE([AC_PADICO_VERSION])
	   AC_REQUIRE([AC_PADICO_COMPILER])
           AC_MSG_CHECKING([for g++ version])
	   if test "x$GXX" != "xyes"; then
	     AC_MSG_ERROR([ $PACKAGE_STRING requires GNU C++ compiler.])
	   else
	     gxx_version="`${CXX} -dumpversion`"
	     case $gxx_version in
	       2.9*|3.0.4|3.1*|3.2*|3.3*|3.4*|4.*)
	         AC_MSG_RESULT([ ${gxx_version} ok])
	       ;;
	       2.*|3.0*)
	         AC_MSG_RESULT([ ${gxx_version} --warning: g++ version not supported. No CORBA support])
	       ;;
	       *)
	         AC_MSG_RESULT([ ${gxx_version} unkown --report how it works!])
	       ;;
	     esac
	   fi
         ])

AC_DEFUN([AC_PADICO_GXX_ABI],
	 [ AC_REQUIRE([AC_PADICO_GXX_VERSION])
	   AC_MSG_CHECKING([for g++ ABI version])
	   case ${gxx_version} in
	     2.*|3.0*|3.1*)
	       GXX_ABI="old"
	       AC_DEFINE([GXX_ABI_OLD], 1, [g++ uses pre-iso ABI.])
	     ;;
	     3.*|4.*)
	       GXX_ABI="iso"
	       AC_DEFINE([GXX_ABI_ISO], 1, [g++ uses iso ABI.])
	     ;;
	     *)
	       GXX_ABI="iso"
	       AC_DEFINE([GXX_ABI_ISO], 1, [unknown c++ version. Assume iso ABI.])
	     ;;
	   esac
	   AC_MSG_RESULT(${GXX_ABI})
	   AC_SUBST(GXX_ABI)
	])
