
dnl --- Types
dnl --------------------------------------------------------

# ## socklen_t
AC_DEFUN(AC_TYPE_SOCKLEN_T,
[AC_CACHE_CHECK([for socklen_t], ac_cv_type_socklen_t,
[
  AC_TRY_COMPILE(
  [#include <sys/socket.h>],
  [socklen_t len = 42; return len;],
  ac_cv_type_socklen_t=yes,
  ac_cv_type_socklen_t=no)
])
  if test $ac_cv_type_socklen_t = yes; then
    AC_DEFINE(HAVE_SOCKLEN_T, [], [whether socklen_t is defined in sys/socket.h])
  fi
])

# ## off64_t
AC_DEFUN([AC_TYPE_OFF64_T],
	 [ AC_CACHE_CHECK([for off64_t], ac_cv_type_off64_t,
[
  AC_TRY_COMPILE(
  [#define _LARGEFILE64_SOURCE
   #include <sys/types.h>],
  [off64_t off = 42;],
  ac_cv_type_off64_t=yes,
  ac_cv_type_off64_t=no)
])
  if test $ac_cv_type_off64_t = yes; then
    AC_DEFINE(HAVE_OFF64_T, [], [whether off64_t is defined in sys/types.h using _LARGEFILE64_SOURCE])
    largefilecflags=-D_LARGEFILE64_SOURCE
  fi
  AC_SUBST(largefilecflags)
])

# ## check base required types
AC_DEFUN([AC_PADICO_TYPES],
	 [ AC_REQUIRE([AC_PADICO_COMPILER])
	   AC_C_BIGENDIAN
	   AC_CHECK_SIZEOF(int)
	   AC_CHECK_SIZEOF(long)
	   AC_CHECK_SIZEOF(long long)
	   AC_CHECK_SIZEOF(ssize_t)
	   AC_TYPE_SOCKLEN_T
	   AC_TYPE_OFF_T
	   AC_TYPE_OFF64_T
	   AC_TYPE_SIZE_T
	   AC_TYPE_PID_T
	 ])

dnl --- libs
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_LIBS],
	 [ AC_REQUIRE([AC_PADICO_LINKER])
	   AC_SEARCH_LIBS(dlopen,  [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlopen() ]) ])
	   AC_SEARCH_LIBS(dlclose, [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlclose() ]) ])
	   AC_SEARCH_LIBS(dlsym,   [dl dlcompat c c++], [], [ AC_MSG_ERROR([Cannot find dlsym() ]) ])

	   AC_SEARCH_LIBS(sched_yield,   [c rt], [], [ AC_MSG_ERROR([Cannot find sched_yield() ]) ])

	   AC_SEARCH_LIBS(socket, nsl socket, [], [ AC_MSG_ERROR([Cannot find socket() ]) ])
	   AC_SEARCH_LIBS(listen, nsl socket, [], [ AC_MSG_ERROR([Cannot find listen() ]) ])
	   AC_SEARCH_LIBS(accept, nsl socket, [], [ AC_MSG_ERROR([Cannot find accept() ]) ])
	   AC_SEARCH_LIBS(gethostbyname, nsl socket, [], [ AC_MSG_ERROR([Cannot find gethostbyname() ]) ])
   
	 ])


dnl --- Arguments
dnl --------------------------------------------------------

AC_DEFUN([AC_PADICO_OPTIMIZE],
	 [ ENABLE_OPTIMIZE_FLAG="no";
	   AC_ARG_ENABLE(optimize,
	        	 [ --enable-optimize ],
	                 [ if test $enableval = yes; then
	                     ENABLE_OPTIMIZE_FLAG="yes";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_OPTIMIZE_FLAG)
	 ])

AC_DEFUN([AC_PADICO_DEBUG],
	 [ ENABLE_DEBUG_FLAG="no";
	   AC_ARG_ENABLE(debug,
	   	         [ --enable-debug ],
	                 [ if test $enableval = yes; then 
	                     ENABLE_DEBUG_FLAG="yes";
	                   fi
	                 ],
	                 [])
	   AC_SUBST(ENABLE_DEBUG_FLAG)
	])

AC_DEFUN([AC_PADICO_TRACE],
	 [ AC_ARG_ENABLE(trace,
	                 [ --enable-trace ], 
	                 [ if test "x$enableval" = "xyes"; then
	                     AC_DEFINE(PADICO_TRACE_ON, 1, [enable Padico trace system])
	                   fi],
	                 [])
	 ])

AC_DEFUN([AC_PADICO_PUKABI],
	 [ PADICO_ENABLE_PUKABI="yes";
	   AC_ARG_ENABLE(pukabi,
	                 [  --disable-pukabi        disable Puk ABI manager (default: enable) ], 
	                 [ if test "x$enableval" = "xno"; then
	                     PADICO_ENABLE_PUKABI="no";
	                   fi],
	                 [])
          AC_ARG_ENABLE([pukabi-filesystem],
                        [AS_HELP_STRING([--disable-pukabi-filesystem],
                          [disable PukABI's file system virtualization mechanism])],
                        [if test "x$enable_val" != "xyes"; then
                           PADICO_ENABLE_PUKABI_FSYS=no
                         else
                           PADICO_ENABLE_PUKABI_FSYS=yes
                         fi],
                        [PADICO_ENABLE_PUKABI_FSYS=yes])

	  AC_SUBST(PADICO_ENABLE_PUKABI)
	  if test "$PADICO_ENABLE_PUKABI" = "yes"; then
	      AC_DEFINE(PADICO_ENABLE_PUKABI, 1, [enable PukABI])
	  fi

         if test "x$PADICO_ENABLE_PUKABI_FSYS" = "xyes"; then
            AC_DEFINE([PADICO_ENABLE_PUKABI_FSYS], [1],
                      [Define to 1 to enable interception of filesystem-related calls by PukABI])
	 fi	
	 ])

