/*
 * NewMadeleine
 * Copyright (C) 2006 (see AUTHORS file)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */


#ifndef NM_LOG_H
#define NM_LOG_H

#define NM_DISPF(str, ...)	if(!(getenv("NMAD_QUIET") || getenv("PADICO_QUIET")))fprintf(stderr, str, ## __VA_ARGS__)

#ifdef NDEBUG
#define NM_WARN(str, ...)       (void)(0)
#else /* NDEBUG */
#define NM_WARN(str, ...)       padico_warning("nmad (%s)-" str, __TBX_FUNCTION__ , ## __VA_ARGS__)
#endif /* NDEBUG */

#define NM_TRACEF(str, ...)	padico_trace(str "\n", ## __VA_ARGS__)
#define NM_TRACE_VAL(str, val)	NM_TRACEF("%s = %d", str, (int)val)
#define NM_TRACE_PTR(str, ptr)	NM_TRACEF("%s = %p", str, (void*)ptr)
#define NM_TRACE_STR(str, str2)	NM_TRACEF("%s: %s", str, str2)

#define NM_LOGF(str, ...)	NM_TRACEF(str , ## __VA_ARGS__)
#define NM_LOG_IN()		NM_TRACEF("-->")
#define NM_LOG_OUT()		NM_TRACEF("<--")


/* Profiling/post-portem analysis
 */
#ifdef PROFILE_NMAD
#  define FUT_NMAD_CODE			0xfe00

#  define FUT_NMAD_EVENT0_CODE		FUT_NMAD_CODE + 0x00
#  define FUT_NMAD_EVENT1_CODE		FUT_NMAD_CODE + 0x01
#  define FUT_NMAD_EVENT2_CODE		FUT_NMAD_CODE + 0x02
#  define FUT_NMAD_EVENTSTR_CODE	FUT_NMAD_CODE + 0x03

#  define FUT_NMAD_EVENT_CONFIG_CODE		FUT_NMAD_CODE + 0x10
#  define FUT_NMAD_EVENT_DRV_ID_CODE		FUT_NMAD_CODE + 0x11
#  define FUT_NMAD_EVENT_DRV_NAME_CODE		FUT_NMAD_CODE + 0x12
#  define FUT_NMAD_EVENT_CNX_CONNECT_CODE	FUT_NMAD_CODE + 0x13
#  define FUT_NMAD_EVENT_CNX_ACCEPT_CODE	FUT_NMAD_CODE + 0x14
#  define FUT_NMAD_EVENT_NEW_TRK_CODE		FUT_NMAD_CODE + 0x15
#  define FUT_NMAD_EVENT_SND_START_CODE		FUT_NMAD_CODE + 0x16
#  define FUT_NMAD_EVENT_RCV_END_CODE		FUT_NMAD_CODE + 0x17

#  define FUT_NMAD_EVENT_ANNOTATE_CODE		FUT_NMAD_CODE + 0x20

#  define FUT_NMAD_INIT_NIC			FUT_NMAD_CODE + 0x40
#  define FUT_NMAD_INIT_GATE			FUT_NMAD_CODE + 0x41
#  define FUT_NMAD_INIT_CORE			FUT_NMAD_CODE + 0x42
#  define FUT_NMAD_INIT_SCHED			FUT_NMAD_CODE + 0x43

#  define FUT_NMAD_GATE_NEW_INPUT_LIST		FUT_NMAD_CODE + 0x44
#  define FUT_NMAD_GATE_NEW_OUTPUT_LIST		FUT_NMAD_CODE + 0x45


#  define FUT_NMAD_NIC_NEW_INPUT_LIST		FUT_NMAD_CODE + 0x46
#  define FUT_NMAD_NIC_NEW_OUTPUT_LIST		FUT_NMAD_CODE + 0x47

#  define FUT_NMAD_GATE_OPS_CREATE_PACKET	FUT_NMAD_CODE + 0x48
#  define FUT_NMAD_GATE_OPS_INSERT_PACKET	FUT_NMAD_CODE + 0x49
#  define FUT_NMAD_GATE_OPS_IN_TO_OUT		FUT_NMAD_CODE + 0x4a
#  define FUT_NMAD_GATE_OPS_OUT_TO_TRACK	FUT_NMAD_CODE + 0x4b

#  define FUT_NMAD_NIC_OPS_GATE_TO_TRACK	FUT_NMAD_CODE + 0x4c
#  define FUT_NMAD_NIC_OPS_SEND_PACKET		FUT_NMAD_CODE + 0x4d

#  define FUT_NMAD_NIC_RECV_ACK_RNDV		FUT_NMAD_CODE + 0x4e

#  define FUT_NMAD_GATE_OPS_CREATE_CTRL_PACKET	FUT_NMAD_CODE + 0x4f
#  define FUT_NMAD_GATE_OPS_IN_TO_OUT_AGREG	FUT_NMAD_CODE + 0x50
#  define FUT_NMAD_NIC_OPS_TRACK_TO_DRIVER	FUT_NMAD_CODE + 0x51
#  define FUT_NMAD_GATE_OPS_IN_TO_OUT_SPLIT	FUT_NMAD_CODE + 0x52

#  define FUT_NMAD_INIT_NIC_URL			FUT_NMAD_CODE + 0x53

#  define NMAD_EVENT0()					\
 PROF_START						\
      FUT_DO_PROBE0(FUT_NMAD_EVENT0_CODE);		\
 PROF_END

#  define NMAD_EVENT1(a)				\
 PROF_START						\
      FUT_DO_PROBE1(FUT_NMAD_EVENT1_CODE, a);		\
 PROF_END

#  define NMAD_EVENT2(a, b)				\
 PROF_START						\
      FUT_DO_PROBE2(FUT_NMAD_EVENT2_CODE, a, b);	\
 PROF_END

#  define NMAD_EVENTSTR(s, ...)							\
 PROF_START									\
      FUT_DO_PROBESTR(FUT_NMAD_EVENTSTR_CODE, s , ## __VA_ARGS__);		\
 PROF_END

#  define NMAD_EVENT_CONFIG(size, rank)				\
 PROF_START							\
      FUT_DO_PROBE2(FUT_NMAD_EVENT_CONFIG_CODE, size, rank);	\
 PROF_END

#  define NMAD_EVENT_DRV_ID(drv_id)				\
 PROF_START							\
      FUT_DO_PROBE1(FUT_NMAD_EVENT_DRV_ID_CODE, drv_id);	\
 PROF_END

#  define NMAD_EVENT_DRV_NAME(drv_name)				\
 PROF_START							\
      FUT_DO_PROBESTR(FUT_NMAD_EVENT_DRV_NAME_CODE, drv_name);	\
 PROF_END

#  define NMAD_EVENT_CNX_CONNECT(remote_rank, gate_id, drv_id)				\
 PROF_START										\
      FUT_DO_PROBE3(FUT_NMAD_EVENT_CNX_CONNECT_CODE, remote_rank, gate_id, drv_id);	\
 PROF_END

#  define NMAD_EVENT_CNX_ACCEPT(remote_rank, gate_id, drv_id)				\
 PROF_START										\
      FUT_DO_PROBE3(FUT_NMAD_EVENT_CNX_ACCEPT_CODE, remote_rank, gate_id, drv_id);	\
 PROF_END

#  define NMAD_EVENT_NEW_TRK(gate_id, drv_id, trk_id)					\
 PROF_START										\
      FUT_DO_PROBE3(FUT_NMAD_EVENT_NEW_TRK_CODE, gate_id, drv_id, trk_id);		\
 PROF_END

#  define NMAD_EVENT_SND_START(gate_id, drv_id, trk_id, length)				\
 PROF_START										\
      FUT_DO_PROBE4(FUT_NMAD_EVENT_SND_START_CODE, gate_id, drv_id, trk_id, length);	\
 PROF_END

#  define NMAD_EVENT_RCV_END(gate_id, drv_id, trk_id, length)				\
 PROF_START										\
      FUT_DO_PROBE4(FUT_NMAD_EVENT_RCV_END_CODE, gate_id, drv_id, trk_id, length);	\
 PROF_END

#  define NMAD_EVENT_ANNOTATE(str, ...)							\
 PROF_START										\
      FUT_DO_PROBESTR(FUT_NMAD_EVENT_ANNOTATE_CODE, str , ## __VA_ARGS__);		\
 PROF_END

#else
#  define NMAD_EVENT0()				(void)0
#  define NMAD_EVENT1(a)			(void)0
#  define NMAD_EVENT2(a, b)			(void)0
#  define NMAD_EVENTSTR(s, ...)			(void)0
#  define NMAD_EVENT_CONFIG(size, rank)		(void)0
#  define NMAD_EVENT_DRV_ID(drv_id)		(void)0
#  define NMAD_EVENT_DRV_NAME(drv_name)		(void)0
#  define NMAD_EVENT_CNX_CONNECT(remote_rank, gate_id, drv_id)		(void)0
#  define NMAD_EVENT_CNX_ACCEPT(remote_rank, gate_id, drv_id)		(void)0
#  define NMAD_EVENT_NEW_TRK(gate_id, drv_id, trk_id)			(void)0
#  define NMAD_EVENT_SND_START(gate_id, drv_id, trk_id, length)		(void)0
#  define NMAD_EVENT_RCV_END(gate_id, drv_id, trk_id, length)		(void)0
#  define NMAD_EVENT_ANNOTATE(str, ...)		(void)0
#  define FUT_DO_PROBE0(a)			(void)0
#  define FUT_DO_PROBE1(a, b)			(void)0
#  define FUT_DO_PROBE2(a, b, c)		(void)0
#  define FUT_DO_PROBE3(a, b, c, d)		(void)0
#  define FUT_DO_PROBE4(a, b, c, d, e)		(void)0
#  define FUT_DO_PROBE5(a, b, c, d, e, f)	(void)0
#  define FUT_DO_PROBESTR(a, str)		(void)0
#endif

#endif /* NM_LOG_H */
